#!/bin/bash
# script to convert a mg5 matrix.f file to unique naming convention
# for use with VINCIA. Also comments out spin average and Bose factors.
mgfile=dummy.f
baseClass=MG5matrix
if [ $# -ge 1 -a "$1" != "-h" -a "$1" != "--help" ]; then
  mgfile=$1
  if [ $# -eq 2 ]; then
    baseClass=$2
  fi 
fi
if [ -r $mgfile ] ; then 
    is=`echo $mgfile | cut -d"_" -f3`
    fs=`echo $mgfile | cut -d"_" -f4 | cut -d"." -f1`
    pro=${is}" > "${fs}
    echo IN: $is   OUT: $fs
#    ndFS=$((`echo $fs | sed 's/[^d]//g'|wc -m`-1))
#    nuFS=$((`echo $fs | sed 's/[^u]//g'|wc -m`-1))
#    nsFS=$((`echo $fs | sed 's/[^s]//g'|wc -m`-1))
#    ncFS=$((`echo $fs | sed 's/[^c]//g'|wc -m`-1))
#    nbFS=$((`echo $fs | sed 's/[^b]//g'|wc -m`-1))
#    ntFS=$((`echo $fs | sed 's/[^t]//g'|wc -m`-1))
#    ngFS=$((`echo $fs | sed 's/[^g]//g'|wc -m`-1))
    name=${is}_${fs}
# Modify MadGraph5 file to be used in VINCIA. 
    date=`date`    
    (echo "C...${name}.f is a part of the VINCIA event generator.";
     echo "C...This file was generated by MADGRAPH 5";
     echo "C...Modified by VINCIA ${date}";
     echo "C...VINCIA is licenced under the GNU GPL version 2.";
     echo "C...Copyright (C) 2018 Peter Skands";
     echo "C...Please respect the work of the MadGraph authors by";
     echo "C...citing MadGraph 5 when this part of VINCIA is used.";
     echo "C...See: https://arxiv.org/abs/1106.0522";
     echo " ";
     # Remove INITIALISE function
     # Force IS_BORN_HEL_SELECTED=TRUE
     # Define POLARIZATIONS as local rather than common-block variable (all -1)
     cat $mgfile | sed /"SUBROUTINE INIT.*"/q | sed s/"      SUBROUTINE INIT.*"/""/ | sed s/'.NOT.IS_BORN_HEL_SELECTED(IHEL)'/'.FALSE.'/ | sed s/'IS_BORN_HEL_SELECTED(IHEL)'/'.TRUE.'/ | sed s/'      LOGICAL IS_BORN_HEL_SELECTED'/'C...Vincia conversion hardcodes for IS_BORN_HEL_SELECTED = .TRUE.\
C      LOGICAL IS_BORN_HEL_SELECTED'/ | sed s/'      COMMON\/BORN_BEAM_POL\/POLARIZATIONS'/'C...Vincia: BORN_BEAM_POL not used \(POLARIZATIONS local and = -1\)\
C      COMMON\/BORN_BEAM_POL\/POLARIZATIONS'/ | sed s/'MATRIXHEL'/'HELMATRIX'/ | sed s/'\_ME'/'MATRIX'/ | sed s/MATRIX\(/MATRIX\(MCMODE\,ICOL\,/g | sed s/'      MATRIX = 0.D0'/'C...VINCIA: Leading Color for Diagonal Entry ICOL\
      MATRIX = CF\(ICOL\,ICOL\)\*JAMP\(ICOL\)\
     \&         \*DCONJG\(JAMP\(ICOL\)\)\/DENOM\(ICOL\)\
C...VINCIA: Absorb subleading color proportional to FSLC = LC\/Sum\(LC\)\
      IF \(MCMODE.EQ.0\) RETURN\
      IF \(MCMODE.EQ.1.AND.MATRIX.LE.ZERO\) RETURN\
        ZLCSUM = 0D0\
        DO I = 1\, NCOLOR\
          ZLCSUM = ZLCSUM + CF\(I\,I\)\*JAMP\(I\)\*DCONJG\(JAMP\(I\)\)\/DENOM\(I\)\
        ENDDO\
C...VINCIA: MCMODE = 2 returns full colour-summed ME\
        IF (MCMODE.EQ.2) MATRIX = ZLCSUM\
C...VINCIA: Fraction of subleading color to absorb into this permutation\
        FSLC = MATRIX\/ZLCSUM\
C...VINCIA: Sum over subleading contributions multiplied by FSLC'/g | sed s/"ZTEMP = ZTEMP"/"IF (I.NE.J) ZTEMP = ZTEMP"/ | sed s/"ZTEMP\*DCONJ"/"FSLC\*ZTEMP\*DCONJ"/ | sed 's/C      CALL GAUGE/      ENDIF\
C      CALL GAUGE/' | sed s/'REAL\*8'/'DOUBLE PRECISION'/ | sed s/"      ANS=ANS\/"/'\
C...VINCIA: Only normalise by Bose factor if doing full colour\
      IF (MCMODE.EQ.2) ANS=ANS\/'/ | sed '/ANS=ANS\*/ i\
C...Cancel helicity-averaging factor when doing helicity amplitudes\
' | sed 's/(ZERO=0D0)/(ZERO=0D0)\
      DOUBLE PRECISION ZLCSUM, FSLC/' | sed s/'PRECISION P(0:3,NEXTERNAL),ANS'/'PRECISION ANS, P(0:3,NEXTERNAL)'/g | sed s/'N P(0:3,NEXTERNAL)'/'N P(0:3,NEXTERNAL)\
C...VINCIA: Define MCMODE and ICOL variables\
C...  MCMODE = 0 : Leading Colour = LC\[icol\]\
C...  MCMODE = 1 : Vincia Colour  = FC \* LC\[icol\]\/sum\(LC\)\
C...  MCMODE = 2 : Full Colour    = FC \(-\> normal MG mode\)\
      INTEGER MCMODE, ICOL'/g | sed s/', P(0:3,NEXTERNAL)'/', P(0:3,NEXTERNAL)\
C...VINCIA: Define MCMODE and ICOL variables\
C...  MCMODE = 0 : Leading Colour = LC\[icol\]\
C...  MCMODE = 1 : Vincia Colour  = FC \* LC\[icol\]\/sum\(LC\)\
C...  MCMODE = 2 : Full Colour    = FC \(-\> normal MG mode\)\
      INTEGER MCMODE, ICOL'/g | sed s/MATRIX/$name/g | sed s/'NTRY=NTRY\+1'/'NTRY=0\
C...\(VINCIA: no NTRY optimisation\)'/ | sed s/"      DATA USERHEL"/'C...VINCIA: USERHEL is initialised by MG5interface\
C      DATA USERHEL'/ | sed s/"      CALL UPDATE"/'C...VINCIA: only static MG couplings are interfaced\
C      CALL UPDATE'/)> $name.f
    # Produce C++ wrapper (including base implementations of setHel(),
    # setIcol(), and a dummy version of setPerm().
    source ./mkWrapper.sh $name.f $baseClass 
    # Print what we did
    echo $0: $mgfile
    echo "     " $pro
    echo "output : Modified MG5 FORTRAN ME     : $name.f (leave in this directory)"
    echo "         Source File for C++ Wrapper : $name.cc (leave in this directory)"
    echo "         Header File for C++ Wrapper : $name.h (move to include/VinciaMG5/)"
    echo "Header files must be moved to include/VinciaMG5 directory:"
    echo "       " mv $hFile ../../include/VinciaMG5/
else 
    echo ""
    echo "     " usage: $0 filename.f [baseClass]
    echo ""
    echo $0:" error! $mgfile not found in current directory"
  echo $0:" Place FORTRAN matrix-element file generated with MadGraph5 (output matrix) in"
  echo $0:" this directory and optionally specify a base class for the C++ wrapper to"
  echo $0:" inherit from (default baseClass = MG5matrix). Note that the MG5 output mode 'standalone' may need to"
  echo $0:" be invoked if any new HELAS calls are involved; the corresponding new HELAS"
  echo $0:" routines must then be placed in interfaces/MG5/dhelas. Also check"
  echo $0:" if any new/extended couplings are present in the MG5 Source/MODEL/coupl.inc"
  echo $0:" file as compared with the one in Vincia's interfaces/MG5 directory and merge"
  echo $0:" as needed, updating the init() function in src/MG5interface.cc as well."
fi
