// gg_dxdg.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2018 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users,
//               http://www.montecarlonet.org/GUIDELINES

// This file was created on Thu  8 Jun 2017 15:38:02 CEST by ./convertMG5.sh
// VINCIA C++ wrapper for gg_dxdg.f

#include "VinciaMG5/gg_dxdg.h"

namespace Vincia {

//====================================================================

// Wrapper for the MG5 FORTRAN subroutine itself

extern "C" {
  extern void shelgg_dxdg_(int *cMode, int *ICOL, double P1[5][4], int *HEL, double* ANS);
}

//--------------------------------------------------------------------

// initHelMap(): create map from vector of helicities to IHEL number

void MG5gg_dxdg::initHelMap() {
  // Hardcoded helicity map extracted from .f file
  vector<int> h; h.resize(5);
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 1;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 2;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 3;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=1;  h[4]=1;  hMapSav[h] = 4;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 5;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 6;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 7;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=1;  h[4]=1;  hMapSav[h] = 8;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 9;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 10;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 11;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=1;  h[4]=1;  hMapSav[h] = 12;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 13;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 14;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 15;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=1;  h[4]=1;  hMapSav[h] = 16;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 17;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 18;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 19;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=1;  h[4]=1;  hMapSav[h] = 20;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 21;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 22;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 23;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=1;  h[4]=1;  hMapSav[h] = 24;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 25;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 26;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 27;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=1;  h[4]=1;  hMapSav[h] = 28;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=-1;  h[4]=-1;  hMapSav[h] = 29;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=-1;  h[4]=1;  hMapSav[h] = 30;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=1;  h[4]=-1;  hMapSav[h] = 31;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=1;  h[4]=1;  hMapSav[h] = 32;
}

//--------------------------------------------------------------------

// initColMap(): create map from vector of MG indices to ICOL numbers

void MG5gg_dxdg::initColMap() {
  // Hardcoded colour-index map extracted from .f file
  // Colour traces extracted from MG5 fortran file
  vector< vector<int> > traces; traces.resize(0);
  vector<int> traceNow;
  traceNow.resize(0);
  // ICOL 1
  traceNow.push_back(1);
  traceNow.push_back(2);
  traceNow.push_back(5);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=1;
  traceNow.resize(0);
  // ICOL 2
  traceNow.push_back(1);
  traceNow.push_back(5);
  traceNow.push_back(2);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=2;
  traceNow.resize(0);
  // ICOL 3
  traceNow.push_back(2);
  traceNow.push_back(1);
  traceNow.push_back(5);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=3;
  traceNow.resize(0);
  // ICOL 4
  traceNow.push_back(2);
  traceNow.push_back(5);
  traceNow.push_back(1);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=4;
  traceNow.resize(0);
  // ICOL 5
  traceNow.push_back(5);
  traceNow.push_back(1);
  traceNow.push_back(2);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=5;
  traceNow.resize(0);
  // ICOL 6
  traceNow.push_back(5);
  traceNow.push_back(2);
  traceNow.push_back(1);
  traceNow.push_back(4);
  traceNow.push_back(3);
  cMapSav[traceNow]=6;
}

//------------------------------------------------------------------

// Return vector of all process labels matched by this ME

vector< vector<int> > MG5gg_dxdg::processList() {
  vector< vector<int> > labels; labels.resize(0);
  vector<int> thisLabel;
  thisLabel.resize(0);
  thisLabel.push_back(21);
  thisLabel.push_back(21);
  thisLabel.push_back(-1);
  thisLabel.push_back(1);
  thisLabel.push_back(21);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(21);
  thisLabel.push_back(21);
  thisLabel.push_back(-2);
  thisLabel.push_back(2);
  thisLabel.push_back(21);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(21);
  thisLabel.push_back(21);
  thisLabel.push_back(-3);
  thisLabel.push_back(3);
  thisLabel.push_back(21);
  labels.push_back(thisLabel);
  thisLabel.resize(0);
  thisLabel.push_back(21);
  thisLabel.push_back(21);
  thisLabel.push_back(-4);
  thisLabel.push_back(4);
  thisLabel.push_back(21);
  labels.push_back(thisLabel);
  // Sanity check
  for (int i=0; i<(int)labels.size(); ++i) {
    if (nLegs() != (int)labels[i].size())
      cout<<"  MG5${procName}::processList(): ERROR! inconsistent number of legs"<<endl;
  }
  return labels;
}

//------------------------------------------------------------------

// Wrapper for MG5 ME, with variables set by the other methods

double MG5gg_dxdg::getME2(int mcMode) {
  // Call wrapped matrix element
  double ans;
  shelgg_dxdg_(&mcMode, &icolSav, pSav, &ihelSav, &ans);
  return ans;
}

} // End namespace Vincia

